@@ -4,6 +4,8 @@ Configurations
 
 from dataclasses import dataclass
 
+from python2verilog.utils import env
+
 
 @dataclass
 class TestbenchConfig:
@@ -20,3 +22,6 @@ class CodegenConfig(TestbenchConfig):
     """
     Configurations for code generator
     """
+
+    # Enable debug comments graph elements as comment
+    add_debug_comments: bool = env.get_var(env.Vars.DEBUG_COMMENTS)
