@@ -9,7 +9,6 @@ import ast
 import copy
 import io
 import logging
-import warnings
 from dataclasses import dataclass, field
 from types import FunctionType
 from typing import Any, Optional, Sequence, Union
@@ -21,7 +20,13 @@ from python2verilog.ir.instance import Instance
 from python2verilog.ir.signals import ProtocolSignals
 from python2verilog.utils.env import is_debug_mode
 from python2verilog.utils.generics import GenericReprAndStr
-from python2verilog.utils.typed import guard_dict, typed, typed_list, typed_strict
+from python2verilog.utils.typed import (
+    guard,
+    guard_dict,
+    typed,
+    typed_list,
+    typed_strict,
+)
 
 
 @dataclass
@@ -41,16 +46,16 @@ class Context(GenericReprAndStr):
     py_string: Optional[str] = None
     _py_ast: Optional[ast.FunctionDef] = None
 
-    input_types: list[type[Any]] = field(default_factory=list)
-    output_types: list[type[Any]] = field(default_factory=list)
+    input_types: Optional[list[type[Any]]] = None
+    output_types: Optional[list[type[Any]]] = None
 
     optimization_level: int = -1
 
     mode: Modes = Modes.NO_WRITE
 
     _global_vars: list[Var] = field(default_factory=list)
-    _input_vars: list[Var] = field(default_factory=list)
-    _output_vars: list[ExclusiveVar] = field(default_factory=list)
+    _input_vars: Optional[list[Var]] = None
+    _output_vars: Optional[list[ExclusiveVar]] = None
     _states: set[str] = field(default_factory=set)
 
     signals: ProtocolSignals = ProtocolSignals(
@@ -70,6 +75,18 @@ class Context(GenericReprAndStr):
     namespace: dict[str, Context] = field(default_factory=dict)  # callable functions
     instances: dict[str, Instance] = field(default_factory=dict)  # generator instances
 
+    @classmethod
+    def empty_valid(cls):
+        """
+        Creates an empty but valid context for testing purposes
+        """
+        cxt = cls()
+        cxt.input_types = []
+        cxt.input_vars = []
+        cxt.output_types = []
+        cxt.output_vars = []
+        return cxt
+
     @property
     def testbench_name(self) -> str:
         """
@@ -81,7 +98,7 @@ class Context(GenericReprAndStr):
         """
         Avoids recursion on itself
         """
-        dic = self.__dict__
+        dic = copy.deepcopy(self.__dict__)
         del dic["namespace"]
         return dic
 
@@ -101,8 +118,8 @@ class Context(GenericReprAndStr):
         assert isinstance(self.py_ast, ast.FunctionDef), self
         assert isinstance(self.py_func, FunctionType), self
 
-        def check_list(list_: list[Any]):
-            return isinstance(list_, list) and len(list_) > 0
+        def check_list(list_: Union[list[Any], None]):
+            return isinstance(list_, list)
 
         if not check_list(self.input_types):
 
@@ -116,7 +133,7 @@ class Context(GenericReprAndStr):
                     return type(0)
                 raise TypeError(f"{ast.dump(arg)}")
 
-            logging.info(f"Using type hints of {self.name} for input types")
+            logging.info("Using type hints of %s for input types", self.name)
             input_args: list[ast.arg] = self.py_ast.args.args
             assert isinstance(input_args, list), f"{ast.dump(self.py_ast)}"
             self.input_types = list(map(input_mapper, input_args))
@@ -133,7 +150,7 @@ class Context(GenericReprAndStr):
                     return type(0)
                 raise TypeError(f"{ast.dump(arg)}")
 
-            logging.info(f"Using type hints of {self.name} for return types")
+            logging.info("Using type hints of %s for return types", self.name)
             output_args: list[ast.arg]
             if isinstance(self.py_ast.returns, ast.Subscript):
                 assert isinstance(self.py_ast.returns.slice, ast.Tuple)
@@ -146,6 +163,7 @@ class Context(GenericReprAndStr):
 
         assert check_list(self.output_types), self
         assert check_list(self.output_vars), self
+        assert isinstance(self._output_vars, list)
 
         assert isinstance(self.optimization_level, int), self
         assert self.optimization_level >= 0, f"{self.optimization_level} {self.name}"
@@ -228,6 +246,7 @@ class Context(GenericReprAndStr):
         """
         Input variables
         """
+        assert guard(self._input_vars, list)
         return copy.deepcopy(self._input_vars)
 
     @input_vars.setter
@@ -239,6 +258,7 @@ class Context(GenericReprAndStr):
         """
         Output variables
         """
+        assert guard(self._output_vars, list), f"Unknown output variables {self}"
         return copy.deepcopy(self._output_vars)
 
     @output_vars.setter
@@ -272,7 +292,7 @@ class Context(GenericReprAndStr):
         var = typed_strict(var, Var)
         if (
             var in self._global_vars
-            or var in self._input_vars
+            or var in self.input_vars
             or var in self.output_vars
         ):
             return
@@ -299,9 +319,9 @@ class Context(GenericReprAndStr):
 
         assert isinstance(name, str)
         variables = [
-            *list(get_strs(self._global_vars)),
-            *list(get_strs(self._input_vars)),
-            *list(get_strs(self._output_vars)),
+            *list(get_strs(self.global_vars)),
+            *list(get_strs(self.input_vars)),
+            *list(get_strs(self.output_vars)),
         ]
         return name in variables
 
@@ -334,12 +354,14 @@ class Context(GenericReprAndStr):
         """
         Checks if input to functions' types matches previous inputs
         """
+        assert guard(self.input_types, list)
         self.__check_types(self.input_types, input_)
 
     def check_output_types(self, output):
         """
         Checks if outputs to functions' types matches previous outputs
         """
+        assert guard(self.output_types, list)
         self.__check_types(self.output_types, output)
 
     def create_instance(self, name: str) -> Instance:
@@ -347,7 +369,6 @@ class Context(GenericReprAndStr):
         Create generator instance
         """
         self.validate()
-
         inst_input_vars: list[Var] = list(
             map(
                 lambda var: ExclusiveVar(f"{name}_{self.name}_{var.py_name}"),
