@@ -76,7 +76,7 @@ def _run_cmd_with_fifos(
             assert process.stderr
             stdout = process.stdout.read()
             stderr = process.stderr.read()
-            logging.debug(f"{e}, {stdout}, {stderr}")
+            logging.debug("%s, %s, %s", e, stdout, stderr)
             return stdout, stderr
 
 
@@ -113,7 +113,7 @@ def _run_cmd_with_files(
             assert process.stderr
             stdout = process.stdout.read()
             stderr = process.stderr.read()
-            logging.debug(f"{e}, {stdout}, {stderr}")
+            logging.debug("%s, %s, %s", e, stdout, stderr)
             return stdout, stderr
 
 
@@ -151,7 +151,7 @@ def run_with_files(
         top_level_module=top_level_module,
         files=input_files.keys(),
     )
-    logging.info(f"Made command `{iverilog_cmd}`")
+    logging.info("Made command `%s`", iverilog_cmd)
     return _run_cmd_with_files(
         command=iverilog_cmd, input_files=input_files, timeout=timeout
     )
